/*This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief An example of usage of the TWI driver library.
 *
 * This file contains an example on how to use the TWI driver library. It sets
 * up the TWI in master mode. Depending on the global definitions READ and
 * WRITE, it will try to read and write data to a TWI slave. See documentation
 * or comments in this file for further information
 *
 * - Compiler:           IAR EWAAVR32 and GNU GCC for AVR32
 * - Supported devices:  All AVR32 devices with a TWI module can be used.
 * - AppNote:            AVR32107 - Using TWI as a master on the AVR32
 *
 * \author               Atmel Corporation: http://www.atmel.com \n
 *                       Support email: avr32@atmel.com
 *
 * $Name:  $
 * $Revision: 1.5 $
 * $RCSfile: twi_example.c,v $
 * $Date: 2006/05/10 11:53:46 $
 **************************************************************************** */


/* ************************************************************************

Copyright (c) 2006, Atmel Corporation All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the
following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. The name of ATMEL may not be used to endorse or promote products
derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY ATMEL ``AS IS'' AND ANY EXPRESS
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE EXPRESSLY AND SPECIFICALLY
DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE

POSSIBILITY OF SUCH DAMAGE.

************************************************************************ */

#include "pio/pio.h"

#include <avr32/sm_100.h>
#include <sys/interrupts.h>

#include <stdio.h>

#include <string.h>

/*
 * The input (switches) header marked J25 must be
 * connected to the header labeled J1 (PORTB[0..7]).
 *
 * The output (leds) header marked J15 must be connected to the header marked J3 (PORTC[0..7])
*/

#define CPU_HZ	20000000

/* This is a work around for the IAR EWAVR32, version 2.10B */
#ifdef __ICCAVR32__
	#define AVR32_TWI_SCL_0_PIN      7
	#define AVR32_TWI_SCL_0_FUNCTION 0
	#define AVR32_TWI_SDA_0_PIN      6
	#define AVR32_TWI_SDA_0_FUNCTION 0
#endif

#define WRITE 0 /* set to 1 for writing */
#define READ  0 /* set to 1 for reading */
#define SUCCESS  0
#define FAILURE -1

/*! \brief Define which adjust the lengt of the delay function */
#define TIMEOUT                 25000

#include "sdram/sdram.h"
#include "sdram/mt481c2m32b2tg.h"
unsigned long sdram_size;

char image_data_buf[4] = {1, 2, 3, 4};

#include "usart/usart.h"
volatile struct avr32_usart_t *usart;
volatile struct avr32_usb_endpoint_t* bulk_endpoint;
volatile struct avr32_usb_endpoint_t* ctrl_endpoint;


// for header file..
void dump_sdram( int start, int count );
//

void delay(int delay)
{
	unsigned int timeout = delay * TIMEOUT;

	do {
		--timeout;
	} while (timeout > 0);
}
/**
 * Print on com port
 */
int print(volatile avr32_usart_t * usart, char *str)
{
	while (*str != '\0')
		usart_putchar(usart, *str++);
	return SUCCESS;
}

void delay_ms( int delay_ms )
{
	delay(delay_ms);
}



int init_usart()
{
	int cpu_hz = 20000000;
	int res;
	struct usart_options_t opt;
	usart = &AVR32_USART1;

	avr32_piomap_t usart_piomap = {				   \
		{AVR32_USART1_RXD_0_PIN, AVR32_USART1_RXD_0_FUNCTION}, \
		{AVR32_USART1_TXD_0_PIN, AVR32_USART1_TXD_0_FUNCTION}   \
	};

  // Set options for the USART
	opt.baudrate = 115200;
	
	opt.charlength = 8;
	opt.paritytype = USART_NO_PARITY;
	opt.stopbits = USART_1_STOPBIT;
	opt.channelmode = USART_NORMAL_CHMODE;

  // Initialize it in RS232 mode
	res = usart_init_rs232(usart, &opt, cpu_hz);
	if ( res != SUCCESS ) return res;

  // Setup pio for USART
	pio_enable_module(usart_piomap, 2);

  // Print a string to the screen
	print(usart, "\n\nUSART initialized!\n");
	
	return SUCCESS;
}

int initControlEP() {
	volatile struct avr32_usb_t* avr32_usb = &AVR32_USB;

	ctrl_endpoint = &avr32_usb->endpoint[0]; // Use endpoint 0 for control
//	volatile struct avr32_usb_ept_cfg_t* ept_cfg_bulk = &ctrl_endpoint->EPT_CFG;
	
	ctrl_endpoint->EPT_CFG.ept_size = AVR32_USB_EPT_SIZE_SZ_64;
	ctrl_endpoint->EPT_CFG.ept_type = AVR32_USB_EPT_TYPE_CONTROL;
	ctrl_endpoint->EPT_CFG.bk_number = AVR32_USB_BK_NUMBER_ONE;
			
	if ( !ctrl_endpoint->EPT_CFG.ept_mapped ) { // failed?
	  print(usart, "USB Init of Endpoint 0 failed..\n");
	  
	  return -1;
	}

	ctrl_endpoint->EPT_CTL_ENB.ept_enable = 1; // enable the endpoint
	
	print( usart, "Init of ctrl endpoint 0 OK \n" );
	
	return SUCCESS;
}


void initUSB() {
	// Set up interrupts
//	intc_register_interrupt(real_usb_tx_int,AVR32_USB_IRQ/32,INT1);
	
	volatile struct avr32_usb_t* avr32_usb = &AVR32_USB;

	avr32_usb->CTRL.en_usb = 1;
	avr32_usb->IEN.end_of_reset = 1;

	initControlEP();

	const volatile avr32_usb_ept_ctl_t* endpoint_control_reg = &ctrl_endpoint->EPT_CTL;
	
	while (1) {
	  endpoint_control_reg->rx_setup ? print(usart, "Setup true\n") : print(usart, "Setup false\n");
	}

	while ( 0 == avr32_usb->INT_STA.end_of_reset) {
	  print( usart, avr32_usb->CTRL.en_usb ? "En_usb: true " : "En_usb: false " );

	  print( usart, avr32_usb->CTRL.faddr_en ? "F_addr_en: true " : "F_addr_en: false " );
	  print( usart, avr32_usb->INT_STA.end_of_reset ? "EOR: true\n" : "EOR: false\n" );
	}
		
	// Enable the USB controller
	avr32_usb->CTRL.en_usb = 1;
	
	print( usart, "USB controller enabled! \n" );

	
	
//	volatile avr32_usb_ept_ctl_enb_t* ept_ctl_enable = &bulk_endpoint->EPT_CTL_ENB;
	

//	ept_ctl_enable->tx_bk_rdy = 1;	

	
	// *** Endpoint 1 ****
	
	bulk_endpoint = &avr32_usb->endpoint[1]; // Use endpoint 1
	volatile struct avr32_usb_ept_cfg_t* ept_cfg_bulk = &bulk_endpoint->EPT_CFG;
	
	ept_cfg_bulk->ept_size = AVR32_USB_EPT_SIZE_SZ_512;
	ept_cfg_bulk->ept_dir = 1; // 0 = Out, 1 = in (seen from the PC!)
	ept_cfg_bulk->ept_type = AVR32_USB_EPT_TYPE_BULK;
	ept_cfg_bulk->bk_number = AVR32_USB_BK_NUMBER_ONE;
			
	if ( !ept_cfg_bulk->ept_mapped ) { // failed?
	  print(usart, "USB Init of endpoint 1 failed..\n");
	}
	
	volatile avr32_usb_ept_ctl_enb_t* ept_ctl_enable = &bulk_endpoint->EPT_CTL_ENB;
	
	ept_ctl_enable->ept_enable = 1; // enable the endpoint
//	ept_ctl_enable->tx_bk_rdy = 1;	
	
	print( usart, "Init of bulk endpoint 1 OK \n" );
}

#define INPUT_MASK 0x00007fff // from image chip 10 data lines + h/v-sync and pixel clock
#define OUTPUT_MASK 0x000000ff


int init() {
	if ( init_usart() != SUCCESS ) return FAILURE;

	volatile struct avr32_usb_t* avr32_usb = &AVR32_USB;
	const volatile avr32_usb_ept_ctl_t* endpoint_control_reg = &ctrl_endpoint->EPT_CTL;

	while (1) {
	  print( usart, avr32_usb->CTRL.faddr_en ? "F_addr_en: true " : "F_addr_en: false " );
	  print( usart, avr32_usb->CTRL.en_usb ? "En_usb: true " : "En_usb: false " );
	  print( usart, avr32_usb->INT_STA.end_of_reset ? "EOR: true\n" : "EOR: false\n" );

	  endpoint_control_reg->rx_setup ? print(usart, "Before init setup true\n") : print(usart, "Before init setup false\n");


		if (!avr32_usb->CTRL.faddr_en) {

		  initUSB();
		  do {
		    endpoint_control_reg->rx_setup ? print(usart, "Setup true\n") : print(usart, "Setup false\n");

	  print( usart, avr32_usb->CTRL.faddr_en ? "F_addr_en: true " : "F_addr_en: false " );
	  print( usart, avr32_usb->CTRL.en_usb ? "En_usb: true " : "En_usb: false " );
	  print( usart, avr32_usb->INT_STA.end_of_reset ? "EOR: true\n" : "EOR: false\n" );

		    delay(100);
		  } while (1);
		}
		
	}

	return SUCCESS;
}

	
int main ( void )
{
	if ( init() == SUCCESS ) {
		print(usart, "All devices initialized!\n");


		//volatile const avr32_usb_ept_ctl_t* endpoint_ctrl_reg =  &bulk_endpoint->EPT_CTL;
		volatile avr32_usb_ept_set_sta_t* endpoint_set_status_reg = &bulk_endpoint->EPT_SET_STA;
		volatile const avr32_usb_ept_sta_t* endpoint_status_reg = &bulk_endpoint->EPT_STA;

		while(1) {
		  while (0 == endpoint_status_reg->tx_bk_ready); // wait until ready to send
		
		  strcpy( (char *)AVR32_USB_FIFO, "This is a test"); // copy data to the FIFO
		
		  endpoint_set_status_reg->tx_bk_rdy = 1; // ready to send
		
		  delay(100);
		}
		
		return SUCCESS;
	}
	else {
	  print(usart, "One device failed initialized!\n");
	  
	  return FAILURE;
	}
}
