/*
 * Register definitions for the Atmel Image Sensor Interface.
 *
 * Copyright (C) 2006 Atmel Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#ifndef __ASM_AVR32_ISI_H__
#define __ASM_AVR32_ISI_H__

#include <linux/videodev2.h>

/* ISI register offsets */
#define ISI_CR1					0x0000
#define ISI_CR2					0x0004
#define ISI_SR					0x0008
#define ISI_IER					0x000c
#define ISI_IDR					0x0010
#define ISI_IMR					0x0014
#define ISI_PSIZE				0x0020
#define ISI_PCDEF				0x0024
#define ISI_PPFBD				0x0028
#define ISI_CDBA				0x002c
#define ISI_Y2R_SET0				0x0030
#define ISI_Y2R_SET1				0x0034
#define ISI_R2Y_SET0				0x0038
#define ISI_R2Y_SET1				0x003c
#define ISI_R2Y_SET2				0x0040

/* Bitfields in CR1 */
#define ISI_RST_OFFSET				0
#define ISI_RST_SIZE				1
#define ISI_DIS_OFFSET				1
#define ISI_DIS_SIZE				1
#define ISI_HSYNC_POL_OFFSET			2
#define ISI_HSYNC_POL_SIZE			1
#define ISI_VSYNC_POL_OFFSET			3
#define ISI_VSYNC_POL_SIZE			1
#define ISI_PIXCLK_POL_OFFSET			4
#define ISI_PIXCLK_POL_SIZE			1
#define ISI_EMB_SYNC_OFFSET			6
#define ISI_EMB_SYNC_SIZE			1
#define ISI_CRC_SYNC_OFFSET			7
#define ISI_CRC_SYNC_SIZE			1
#define ISI_FRATE_OFFSET			8
#define ISI_FRATE_SIZE				3
#define ISI_FULL_OFFSET				12
#define ISI_FULL_SIZE				1
#define ISI_THMASK_OFFSET			13
#define ISI_THMASK_SIZE				2
#define ISI_CODEC_ON_OFFSET			15
#define ISI_CODEC_ON_SIZE			1
#define ISI_SLD_OFFSET				16
#define ISI_SLD_SIZE				8
#define ISI_SFD_OFFSET				24
#define ISI_SFD_SIZE				8

/* Bitfields in CR2 */
#define ISI_IM_VSIZE_OFFSET			0
#define ISI_IM_VSIZE_SIZE			11
#define ISI_GS_MODE_OFFSET			11
#define ISI_GS_MODE_SIZE			1
#define ISI_RGB_MODE_OFFSET			12
#define ISI_RGB_MODE_SIZE			1
#define ISI_GRAYSCALE_OFFSET			13
#define ISI_GRAYSCALE_SIZE			1
#define ISI_RGB_SWAP_OFFSET			14
#define ISI_RGB_SWAP_SIZE			1
#define ISI_COL_SPACE_OFFSET			15
#define ISI_COL_SPACE_SIZE			1
#define ISI_IM_HSIZE_OFFSET			16
#define ISI_IM_HSIZE_SIZE			11
#define ISI_YCC_SWAP_OFFSET			28
#define ISI_YCC_SWAP_SIZE			2
#define ISI_RGB_CFG_OFFSET			30
#define ISI_RGB_CFG_SIZE			2

/* Bitfields in SR */
#define ISI_CDC_STATUS_OFFSET			3
#define ISI_CDC_STATUS_SIZE			1

/* Bitfields in SR/IER/IDR/IMR */
#define ISI_SOF_OFFSET				0
#define ISI_SOF_SIZE				1
#define ISI_SOFTRST_OFFSET			2
#define ISI_SOFTRST_SIZE			1
#define ISI_CRC_ERR_OFFSET			4
#define ISI_CRC_ERR_SIZE			1
#define ISI_FO_C_OVF_OFFSET			5
#define ISI_FO_C_OVF_SIZE			1
#define ISI_FO_P_OVF_OFFSET			6
#define ISI_FO_P_OVF_SIZE			1
#define ISI_FO_P_EMP_OFFSET			7
#define ISI_FO_P_EMP_SIZE			1
#define ISI_FO_C_EMP_OFFSET			8
#define ISI_FO_C_EMP_SIZE			1
#define ISI_FR_OVR_OFFSET			9
#define ISI_FR_OVR_SIZE				1

/* Bitfields in PSIZE */
#define ISI_PREV_VSIZE_OFFSET			0
#define ISI_PREV_VSIZE_SIZE			10
#define ISI_PREV_HSIZE_OFFSET			16
#define ISI_PREV_HSIZE_SIZE			10

/* Bitfields in PCDEF */
#define ISI_DEC_FACTOR_OFFSET			0
#define ISI_DEC_FACTOR_SIZE			8

/* Bitfields in PPFBD */
#define ISI_PREV_FBD_ADDR_OFFSET		0
#define ISI_PREV_FBD_ADDR_SIZE			32

/* Bitfields in CDBA */
#define ISI_CODEC_DMA_ADDR_OFFSET		0
#define ISI_CODEC_DMA_ADDR_SIZE			32

/* Bitfields in Y2R_SET0 */
#define ISI_Y2R_SET0_C3_OFFSET			24
#define ISI_Y2R_SET0_C3_SIZE			8

/* Bitfields in Y2R_SET1 */
#define ISI_Y2R_SET1_C4_OFFSET			0
#define ISI_Y2R_SET1_C4_SIZE			9
#define ISI_YOFF_OFFSET				12
#define ISI_YOFF_SIZE				1
#define ISI_CROFF_OFFSET			13
#define ISI_CROFF_SIZE				1
#define ISI_CBOFF_OFFSET			14
#define ISI_CBOFF_SIZE				1

/* Bitfields in R2Y_SET0 */
#define ISI_C0_OFFSET				0
#define ISI_C0_SIZE				8
#define ISI_C1_OFFSET				8
#define ISI_C1_SIZE				8
#define ISI_C2_OFFSET				16
#define ISI_C2_SIZE				8
#define ISI_ROFF_OFFSET				24
#define ISI_ROFF_SIZE				1

/* Bitfields in R2Y_SET1 */
#define ISI_R2Y_SET1_C3_OFFSET			0
#define ISI_R2Y_SET1_C3_SIZE			8
#define ISI_R2Y_SET1_C4_OFFSET			8
#define ISI_R2Y_SET1_C4_SIZE			8
#define ISI_C5_OFFSET				16
#define ISI_C5_SIZE				8
#define ISI_GOFF_OFFSET				24
#define ISI_GOFF_SIZE				1

/* Bitfields in R2Y_SET2 */
#define ISI_C6_OFFSET				0
#define ISI_C6_SIZE				8
#define ISI_C7_OFFSET				8
#define ISI_C7_SIZE				8
#define ISI_C8_OFFSET				16
#define ISI_C8_SIZE				8
#define ISI_BOFF_OFFSET				24
#define ISI_BOFF_SIZE				1

/* Constants for FRATE */
#define ISI_FRATE_CAPTURE_ALL			0

/* Constants for YCC_SWAP */
#define ISI_YCC_SWAP_DEFAULT			0
#define ISI_YCC_SWAP_MODE_1			1
#define ISI_YCC_SWAP_MODE_2			2
#define ISI_YCC_SWAP_MODE_3			3

/* Constants for RGB_CFG */
#define ISI_RGB_CFG_DEFAULT			0
#define ISI_RGB_CFG_MODE_1			1
#define ISI_RGB_CFG_MODE_2			2
#define ISI_RGB_CFG_MODE_3			3

/* Bit manipulation macros */
#define ISI_BIT(name)					\
	(1 << ISI_##name##_OFFSET)
#define ISI_BF(name,value)				\
	(((value) & ((1 << ISI_##name##_SIZE) - 1))	\
	 << ISI_##name##_OFFSET)
#define ISI_BFEXT(name,value)				\
	(((value) >> ISI_##name##_OFFSET)		\
	 & ((1 << ISI_##name##_SIZE) - 1))
#define ISI_BFINS(name,value,old)			\
	(((old) & ~(((1 << ISI_##name##_SIZE) - 1)	\
		    << ISI_##name##_OFFSET))\
	 | ISI_BF(name,value))

/* Register access macros */
#define isi_readl(port,reg)				\
	__raw_readl((port)->regs + ISI_##reg)
#define isi_writel(port,reg,value)			\
	__raw_writel((value), (port)->regs + ISI_##reg)

#define ATMEL_V4L2_VID_FLAGS ( V4L2_CAP_VIDEO_OUTPUT )

struct avr32_v4l2_ops{
  u32 frate:3;

  u32 im_hsize:10;
  u32 im_vsize:10;

  u32 gs:1;
  u32 gs_mode:1; // Marius change: 2 pixels / word
  u32 col_space:1;

  u32 rgb_mode:1;
  u32 rgb_cfg:2;

  u32 rgb_swap:2;
  u32 ycc_swap:2;
  u32 mclk;
  u32 format;
  u32 type;
};

struct avr32_4vl2_dev{
  u32 hsync_pol:1;
  u32 vsync_pol:1;
  u32 pixclk_pol:1;

  u32 emb_sync:1;
  u32 crc_sync:1;

  u32 full:1;
  u32 thmask:2;
  u32 codec_on:1;

  u32 sld:8;
  u32 sfd:8;

  u32 bayer:1;
};

struct atmel_isi;

struct atmel_isi_format {
	struct v4l2_pix_format pix;
	unsigned int bytes_per_pixel;
};

struct atmel_isi_camera {
	const char		*name;
	struct module		*owner;
	struct list_head	list;
	unsigned int		hsync_act_low:1;
	unsigned int		vsync_act_low:1;
	unsigned int		pclk_act_falling:1;
	int (*enum_format)(struct atmel_isi_camera *cam,
			   struct v4l2_fmtdesc *fmt);
	int (*get_format)(struct atmel_isi_camera *cam,
			  struct atmel_isi_format *fmt);
	int (*set_format)(struct atmel_isi_camera *cam,
			  struct atmel_isi_format *fmt);
	int (*start_capture)(struct atmel_isi_camera *cam);
	int (*stop_capture)(struct atmel_isi_camera *cam);
	struct atmel_isi	*isi;
};

extern int atmel_isi_register_camera(struct atmel_isi_camera *cam);
extern void atmel_isi_unregister_camera(struct atmel_isi_camera *cam);

/*
 *
 * VIDEO DEVICES
 *
 */
#if 0
#ifdef CONFIG_VIDEO_AVR32_ISI_AT76C451BC
#define ATMEL_ISI_DEVNAME "AT76C451BC"

struct avr32_v4l2_ops cam_ops =
{
        .frate = 0,		/* all frames are captured. If set each {frate+1} frame will be captured*/

        .im_hsize = 320,	/* img horizontal size in pix */
        .im_vsize = 240,	/* img vertical size in pix */

        .gs = 0,		/* grayscale is disabled */
        .gs_mode = 0,		/* not used if gs is disabled */

        .col_space = 1,		/* 0-> YCbCr, 1 -> RGB */

        .rgb_mode = 0,		/* 0-> 24 bits, 1 -> 16 bits */
        .rgb_cfg = 0,		/* See chapter 37.5.2 in avr32ap7000 datasheet */

        .rgb_swap = 0,		/* 0 -> D7->R7, 1 -> D0 -> R7 */
        .ycc_swap = 0,		/* See chapter 37.5.2 in avr32ap7000 datasheet */

	.mclk = 20000000,
	.format =V4L2_PIX_FMT_RGB24,
	.type = VID_TYPE_CAPTURE,
  };
struct avr32_4vl2_dev cam_dev =
  {
	.hsync_pol = 0,		/* 0 -> active high, 1 -> active low */
	.vsync_pol = 0,		/* 0 -> active high, 1 -> active low */
	.pixclk_pol = 0,	/* 0 -> active high, 1 -> active low */

	.emb_sync = 0,		/* 0 -> hsync & vsync, 1 -> embedded sync sequence by SAV/EAV */
	.crc_sync = 0,		/* 0 -> no crc check, 1 -> crc correction enabled */

	.full = 1,		/* 1-> both preview and codec datapath are enabled */
	.thmask = 0,		/* 0 -> 0,4,8,16, 1 -> 8,16, 2-> 16. Length of allowed System Bus bursts */
	.codec_on = 0,		/* Codec datapath disabled/enabled */

	.sld = 0,		/* NO clock periods to wait before beginning of a new line */
	.sfd = 0,		/* NO skipped lines skipped at the beginning of the frame */
};

#elif CONFIG_VIDEO_AVR32_ISI_MT9V011
#define ATMEL_ISI_DEVNAME "MT9V011"
struct avr32_v4l2_ops cam_ops =
{
        .frate = 0,		/* all frames are captured. If set each {frate+1} frame will be captured*/

        .im_hsize = 320 / 2,	/* img horizontal size in pix */
        .im_vsize = 240,	/* img vertical size in pix */

        .gs = 0,		/* grayscale is disabled */
        .gs_mode = 0,		/* not used if gs is disabled */

        .col_space = 1,		/* 0-> YCbCr, 1 -> RGB */

        .rgb_mode = 1,		/* 0-> 24 bits, 1 -> 16 bits */
        .rgb_cfg = 0,		/* See chapter 37.5.2 in avr32ap7000 datasheet */

        .rgb_swap = 0,		/* 0 -> D7->R7, 1 -> D0 -> R7 */
        .ycc_swap = 0,		/* See chapter 37.5.2 in avr32ap7000 datasheet */

	.mclk = 20000000,
	.format = V4L2_PIX_FMT_SBGGR8,
	.type = VID_TYPE_CAPTURE,
};
struct avr32_4vl2_dev cam_dev =
{
	.hsync_pol = 0,		/* 0 -> active high, 1 -> active low */
	.vsync_pol = 1,		/* 0 -> active high, 1 -> active low */
	.pixclk_pol = 0,	/* 0 -> active high, 1 -> active low */

	.emb_sync = 0,		/* 0 -> hsync & vsync, 1 -> embedded sync sequence by SAV/EAV */
	.crc_sync = 0,		/* 0 -> no crc check, 1 -> crc correction enabled */

	.full = 0,		/* 1-> both preview and codec datapath are enabled */
	.thmask = 0,		/* 0 -> 0,4,8,16, 1 -> 8,16, 2-> 16. Length of allowed System Bus bursts */
	.codec_on = 1,		/* Codec datapath disabled/enabled */

	.sld = 0x83,		/* NO clock periods to wait before beginning of a new line */
	.sfd = 0x1C,		/* NO skipped lines skipped at the beginning of the frame */
};

#elif CONFIG_VIDEO_MT9T001
#define ATMEL_ISI_DEVNAME "MT9T001"
struct avr32_v4l2_ops cam_ops =
{
        .frate = 0,		/* all frames are captured. If set each {frate+1} frame will be captured*/

	//        .im_hsize = 320 / 2,	/* img horizontal size in pix */
        //.im_vsize = 240,	/* img vertical size in pix */ 

        .im_hsize = 1024,	/* img horizontal size in pix */
        .im_vsize = 768,	/* img vertical size in pix */


        .gs = 1,		/* grayscale is enabled */
        .gs_mode = 0,		/* 2 pixel pr. word */   // Marius change: 2 pixels / word

        .col_space = 1,		/* 0-> YCbCr, 1 -> RGB */

        .rgb_mode = 1,		/* 0-> 24 bits, 1 -> 16 bits */
        .rgb_cfg = 0,		/* See chapter 37.5.2 in avr32ap7000 datasheet */

        .rgb_swap = 0,		/* 0 -> D7->R7, 1 -> D0 -> R7 */
        .ycc_swap = 0,		/* See chapter 37.5.2 in avr32ap7000 datasheet */

	.mclk = 20000000,
	.format = V4L2_PIX_FMT_SBGGR8,
	.type = VID_TYPE_CAPTURE,
};
struct avr32_4vl2_dev cam_dev =
{
	.hsync_pol = 0,		/* 0 -> active high, 1 -> active low */
	.vsync_pol = 1,		/* 0 -> active high, 1 -> active low */
	.pixclk_pol = 0,	/* 0 -> active high, 1 -> active low */

	.emb_sync = 0,		/* 0 -> hsync & vsync, 1 -> embedded sync sequence by SAV/EAV */
	.crc_sync = 0,		/* 0 -> no crc check, 1 -> crc correction enabled */

	.full = 0,		/* 1-> both preview and codec datapath are enabled */
	.thmask = 0,		/* 0 -> 0,4,8,16, 1 -> 8,16, 2-> 16. Length of allowed System Bus bursts */
	.codec_on = 1,		/* Codec datapath disabled/enabled */

	.sld = 0x83,		/* NO clock periods to wait before beginning of a new line */
	.sfd = 0x1C,		/* NO skipped lines skipped at the beginning of the frame */
};



#else
#define ATMEL_ISI_DEVNAME "atmel isi video device"
struct avr32_v4l2_ops cam_ops =
{
        .frate = 0,		/* all frames are captured. If set each {frate+1} frame will be captured*/
        .im_hsize = 0,		/* img horizontal size in pix */
        .im_vsize = 0,		/* img vertical size in pix */
        .gs = 0,		/* grayscale is disabled */
        .gs_mode = 0,		/* not used if gs is disabled */
        .col_space = 0,		/* 0-> YCbCr, 1 -> RGB */
        .rgb_mode = 0,		/* 0-> 24 bits, 1 -> 16 bits */
        .rgb_cfg = 0,		/* See chapter 37.5.2 in avr32ap7000 datasheet */
        .rgb_swap = 0,		/* 0 -> D7->R7, 1 -> D0 -> R7 */
        .ycc_swap = 0,		/* See chapter 37.5.2 in avr32ap7000 datasheet */
	.mclk = 20000000,	/* Set isi mclk to 20 MHz */
	.format = V4L2_PIX_FMT_RGB24,
  };

struct avr32_4vl2_dev cam_dev =
{
	.hsync_pol = 0,		/* 0 -> active high, 1 -> active low */
	.vsync_pol = 0,		/* 0 -> active high, 1 -> active low */
	.pixclk_pol = 0,	/* 0 -> active high, 1 -> active low */
	.emb_sync = 0,		/* 0 -> hsync & vsync, 1 -> embedded sync sequence by SAV/EAV */
	.crc_sync = 0,		/* 0 -> no crc check, 1 -> crc correction enabled */
	.full = 0,		/* 1-> both preview and codec datapath are enabled */
	.thmask = 0,		/* 0 -> 0,4,8,16, 1 -> 8,16, 2-> 16. Length of allowed System Bus bursts */
	.codec_on = 0,		/* Codec datapath disabled/enabled */

	.sld = 0,		/* NO clock periods to wait before beginning of a new line */
	.sfd = 0,		/* NO skipped lines skipped at the beginning of the frame */
};

#endif /* camera selection */
#endif

#endif /* __ASM_AVR32_ISI_H__ */

