/*
 * Micron MT9V011 Image Sensor driver
 *
 * Copyright (C) 2006 Atmel Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#define DEBUG
#include <linux/clk.h>
#include <linux/err.h>
#include <linux/i2c.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/slab.h>

#include "atmel-isi.h"
#include "mt9t001.h"

/* TODO: Part of this ID is probably a version number */
#define MT9T001_CHIP_ID		0x1621

#define MT9T001_MAX_WIDTH	2048
#define MT9T001_MAX_HEIGHT	1536

#define I2C_MT9T001		0x5d

static unsigned short normal_i2c[] = {
	I2C_MT9T001,
	I2C_CLIENT_END
};
I2C_CLIENT_INSMOD;

#ifdef CONFIG_DEBUG_FS
struct reg_dbg {
	struct mt9t001 *is;
	struct dentry *dentry;
	unsigned int offset;
};
#endif

struct mt9t001 {
	struct i2c_client	client;
	struct atmel_isi_camera	cam;
#ifdef CONFIG_DEBUG_FS
	struct dentry *debugfs_root;
	struct reg_dbg debugfs_reg[0x100];
#endif
};
#define to_mt9t001(cam) container_of(cam, struct mt9t001, cam)

static struct i2c_driver mt9t001_driver;

struct mt9t001 *is_global;

static int mt9t001_read(struct mt9t001 *is, u8 reg)
{
  // Marius test
  	printk( "Skipping mt9t001_write...\n" );
	return 0;
	/*
	int ret;

	ret = i2c_smbus_read_word_data(&is->client, reg);
	if (ret < 0)
		return ret;
		return le16_to_cpu(ret); */
}

static int mt9t001_write(struct mt9t001 *is, u8 reg, u16 value)
{
  // Marius test
  	printk( "Skipping mt9t001_write...\n" );
	return 0;
	//

	//	return i2c_smbus_write_word_data(&is->client, reg,
	//					 cpu_to_le16(value));
}

#ifdef CONFIG_DEBUG_FS
#include <linux/debugfs.h>
#include <linux/uaccess.h>

static const char * reg_name[0x100] = {
	[REG_CHIP_VERSION]	= "chip_version",
	[REG_ROW_START]		= "row_start",
	[REG_COL_START]		= "col_start",
	[REG_WIN_HEIGHT]	= "win_height",
	[REG_WIN_WIDTH]		= "win_width",
	[REG_HBLANK]		= "hblank",
	[REG_VBLANK]		= "vblank",
	[REG_OUTPUT_CTRL]	= "output_ctrl",
	[REG_SHUTTER_WIDTH_UPPER] = "shutter_width_upper",
	[REG_SHUTTER_WIDTH]	= "shutter_width",

	[REG_PIXCLK]		= "pixclk",
	[REG_RESTART]		= "restart",
	[REG_SHUTTER_DELAY]	= "shutter_delay",
	[REG_RESET]		= "reset",

	// end common 

	[REG_READ_MODE1]	= "read_mode1",
	[REG_READ_MODE2]	= "read_mode2",
	[REG_READ_MODE3]	= "read_mode3",

	[REG_ROW_ADDRESS_MODE]	= "row_address_mode",
	[REG_COLUMN_ADDRESS_MODE] = "column_address_mode",

	[REG_GREEN1_GAIN]	= "green1_gain",
	[REG_BLUE_GAIN]		= "blue_gain",
	[REG_RED_GAIN]		= "red_gain",
	[REG_GREEN2_GAIN]	= "green2_gain",

	[REG_TEST_DATA]	        = "test_data",
	[REG_GLOBAL_GAIN]	= "global_gain",

	[REG_BLACK_LEVEL]	= "black_level",
	[REG_BLACK_DEF_OFFSET]	= "black_def_offset",
	[REG_BLC_DELTA_THRES]	= "blc_delta_thres",
	[REG_CAL_THRES]	        = "cal_thres",

	[REG_GREEN1_OFFSET]	= "green1_offset",
	[REG_GREEN2_OFFSET]	= "green2_offset",

	[REG_BLACK_LEVEL_CAL]	= "black_level_cal",

	[REG_RED_OFFSET]	= "red_offset",
	[REG_BLUE_OFFSET]	= "blue_offset",
	[REG_CHIP_ENABLE]	= "chip_enable",
};

static u64 reg_dbg_get(void *data)
{
	struct reg_dbg *reg = data;
	int ret;

	ret = mt9t001_read(reg->is, reg->offset);
	if (ret < 0) {
		printk("%s: failed to read reg 0x%02x: %d\n",
		       reg->is->cam.name, reg->offset, ret);
		return ~0ULL;
	}

	return ret;
}

static void reg_dbg_set(void *data, u64 val)
{
	struct reg_dbg *reg = data;
	int ret;

	ret = mt9t001_write(reg->is, reg->offset, val);
	if (ret < 0)
		printk("%s: failed to write reg 0x%02x: %d\n",
		       reg->is->cam.name, reg->offset, ret);
}
DEFINE_SIMPLE_ATTRIBUTE(reg_dbg_fops, reg_dbg_get, reg_dbg_set, "%04llx\n");

static void mt9t001_init_debugfs(struct mt9t001 *is)
{
	struct dentry *root, *reg;
	unsigned int i;

	root = debugfs_create_dir(is->cam.name, NULL);
	if (IS_ERR(root) || !root)
		goto err_root;
	is->debugfs_root = root;

	for (i = 0; i < ARRAY_SIZE(is->debugfs_reg); i++) {
		if (!reg_name[i])
			continue;

		is->debugfs_reg[i].is = is;
		is->debugfs_reg[i].offset = i;

		reg = debugfs_create_file(reg_name[i], S_IRUGO | S_IWUSR,
					  root, &is->debugfs_reg[i],
					  &reg_dbg_fops);
		if (!reg)
			goto err_reg;
		is->debugfs_reg[i].dentry = reg;
	}

	return;

err_reg:
	while (i--)
		debugfs_remove(is->debugfs_reg[i].dentry);
	debugfs_remove(root);
err_root:
	is->debugfs_root = NULL;
	printk(KERN_ERR "%s: failed to initialize debugfs\n",
	       is->cam.name);
}

static void mt9t001_cleanup_debugfs(struct mt9t001 *is)
{
	unsigned int i;

	if (is->debugfs_root) {
		for (i = 0; i < ARRAY_SIZE(is->debugfs_reg); i++)
			debugfs_remove(is->debugfs_reg[i].dentry);
		debugfs_remove(is->debugfs_root);
	}
}
#else
static inline void mt9t001_init_debugfs(struct mt9t001 *is)
{

}

static inline void mt9t001_cleanup_debugfs(struct mt9t001 *is)
{

}
#endif /* CONFIG_DEBUG_FS */

static int mt9t001_enum_format(struct atmel_isi_camera *cam,
			       struct v4l2_fmtdesc *fmt)
{
  if (fmt->index != 0) return -EINVAL;
  
  /*  switch (fmt->index) {
      0:*/
	strcpy(fmt->description, "RGB Bayer");
	fmt->pixelformat = V4L2_PIX_FMT_SBGGR8;
	//	break;

	/*    1:
	strcpy(fmt->description, "Raw grayscale");
	fmt->pixelformat = V4L2_PIX_FMT_GREY;
	break;
	*/	
	
	/*  default:
    return -EINVAL;
    }*/
	return 0;
}

static int mt9t001_get_win_height(struct mt9t001 *is)
{
  // return mt9t001_read(is, REG_WIN_HEIGHT);

  // Marius hack as i2c not working, not correct but for testing..

  return 1536;
}

static int mt9t001_get_win_width(struct mt9t001 *is)
{
  // Marius hack as i2c not working, not correct but for testing..

  //return mt9t001_read(is, REG_WIN_WIDTH);

  return 2048;
}


static int mt9t001_get_format(struct atmel_isi_camera *cam,
			      struct atmel_isi_format *fmt)
{
	struct mt9t001 *is = to_mt9t001(cam);
	int ret;

	ret = mt9t001_get_win_height(is);
	if (ret < 0)
		return ret;
	fmt->pix.height = ret + 1;

	ret = mt9t001_get_win_width(is);

	if (ret < 0)
		return ret;
	fmt->pix.width = ret + 1;

	fmt->pix.pixelformat = V4L2_PIX_FMT_SBGGR8;
	fmt->pix.colorspace = V4L2_COLORSPACE_SRGB;
	fmt->bytes_per_pixel = 2;

	return 0;
}

static int mt9t001_set_format(struct atmel_isi_camera *cam,
			       struct atmel_isi_format *fmt)
{
	struct mt9t001 *is = to_mt9t001(cam);
	int ret;

	fmt->pix.pixelformat = V4L2_PIX_FMT_SBGGR8;
	fmt->pix.colorspace = V4L2_COLORSPACE_SRGB;
	fmt->bytes_per_pixel = 1;

	if (fmt->pix.height > MT9T001_MAX_HEIGHT)
		fmt->pix.height = MT9T001_MAX_HEIGHT;
	if (fmt->pix.width > MT9T001_MAX_WIDTH)
		fmt->pix.width = MT9T001_MAX_WIDTH;

	ret = mt9t001_write(is, REG_WIN_HEIGHT, fmt->pix.height - 1);
	if (ret)
		return ret;
	ret = mt9t001_write(is, REG_WIN_WIDTH, fmt->pix.width - 1);

	if (!ret)
		pr_debug("%s: set_format %ux%u\n", cam->name,
			 fmt->pix.width, fmt->pix.height);

	return ret;
}

static int mt9t001_start_capture(struct atmel_isi_camera *cam)
{
	struct mt9t001 *is = to_mt9t001(cam);
	int ret;

	ret = mt9t001_write(is, REG_OUTPUT_CTRL, 2);
	if (!ret)
		pr_debug("%s: start_capture (OUTPUT_CTRL=0x%x)\n",
			 cam->name, mt9t001_read(is, REG_OUTPUT_CTRL));
	return ret;
}

static int mt9t001_stop_capture(struct atmel_isi_camera *cam)
{
	struct mt9t001 *is = to_mt9t001(cam);
	int ret;

	ret = mt9t001_write(is, REG_OUTPUT_CTRL, 0);
	if (!ret)
		pr_debug("%s: stop_capture (OUTPUT_CTRL=0x%x)\n",
			 cam->name, mt9t001_read(is, REG_OUTPUT_CTRL));
	return ret;
}

static int mt9t001_init_cam()
{
	struct mt9t001 *is;
	int ret;

	is = kzalloc(sizeof(struct mt9t001), GFP_KERNEL);
	if (!is)
		return -ENOMEM;

	is->cam.name = "mt9t001"; // client->name;
	is->cam.hsync_act_low = 0;
	is->cam.vsync_act_low = 0;
	is->cam.pclk_act_falling = 1;
	is->cam.enum_format = mt9t001_enum_format;
	is->cam.get_format = mt9t001_get_format;
	is->cam.set_format = mt9t001_set_format;
	is->cam.start_capture = mt9t001_start_capture;
	is->cam.stop_capture = mt9t001_stop_capture;

	/* Reset the camera and stop sensor readout. */
	mt9t001_write(is, REG_RESET, 1);
	mt9t001_write(is, REG_RESET, 0);
	mt9t001_write(is, REG_OUTPUT_CTRL, 0);

	mt9t001_write(is, REG_PIXCLK, 0x15); // reduce pixclock by 21*2

	pr_debug("%s: after reset: REG_RESET=0x%x REG_OUTPUT_CTRL=0x%x\n",
		 is->cam.name, mt9t001_read(is, REG_RESET),
		 mt9t001_read(is, REG_OUTPUT_CTRL));

	/* Tell the ISI that we're here */
	ret = atmel_isi_register_camera(&is->cam);
	//	if (ret)
	//		goto err_register;

	printk(KERN_INFO "MT9T001 Image Sensor at %s:0x%02x\n",
	       /*adapter->name*/ "hardcoda: twi" , 0 /*address*/);

	mt9t001_init_debugfs(is);

	is_global = is;

	return 0;
}

static int mt9t001_detect_client(struct i2c_adapter *adapter,
				 int address, int kind)
{
	struct i2c_client *client;

	int chip_id;
	int ret;

	pr_debug("mt9t001: detecting client on address 0x%x\n", address);

	/* Check if the adapter supports the needed features */
	if (!i2c_check_functionality(adapter,
				     (I2C_FUNC_SMBUS_READ_WORD_DATA
				      | I2C_FUNC_SMBUS_WRITE_WORD_DATA)))
		return 0;

	// TODO:problem	client = &is->client;
	client->addr = address;
	client->adapter = adapter;
	client->driver = &mt9t001_driver;
	strcpy(client->name, "mt9t001");


	// Marius hack
	// ret = i2c_attach_client(client);
	//	if (ret)
	//		goto err_attach;

	printk( "Skipping identifying of camera...\n" );
	/* Try to identify the camera

 */
	/*	chip_id = mt9t001_read(is, REG_CHIP_VERSION);
	if (chip_id != MT9T001_CHIP_ID) {
		printk(KERN_ERR "mt9t001: Unknown chip ID 0x%04x\n",
		       chip_id);
	*/
		// Marius hack
	//		printk( "Continuing anyway...\n" );
		/*		ret = -ENODEV;
		  goto err_identify;*/
	//	}

	// Marius hack
	// i2c_set_clientdata(client, is);

	return mt9t001_init_cam();

	//	return 0;

err_register:
err_identify:
	i2c_detach_client(client);
err_attach:
	// TODO: problem	kfree(is);
	return ret;
}

static int mt9t001_attach_adapter(struct i2c_adapter *adapter)
{
	pr_debug("mt9001: starting probe for adapter %s (%u)\n",
		 adapter->name, adapter->id);
	return i2c_probe(adapter, &addr_data, &mt9t001_detect_client);
}

static void mt9t001_cleanup( struct mt9t001 *is)
{
	mt9t001_cleanup_debugfs(is);

	atmel_isi_unregister_camera(&is->cam);

	kfree(is);
}

static int mt9t001_detach_client(struct i2c_client *client)
{
	struct mt9t001 *is = i2c_get_clientdata(client);
	int ret;

	ret = i2c_detach_client(client);
	if (ret)
		return ret;

	return 0;
}

static struct i2c_driver mt9t001_driver = {
	.driver = {
		.name	= "mt9t001",
	},
	.id		= I2C_DRIVERID_MT9T001,
	.attach_adapter	= &mt9t001_attach_adapter,
	.detach_client	= &mt9t001_detach_client,
};

static struct clk *mclk;

static int __init mt9t001_init(void)
{
	/*
	 * Enable the master clock. If clk_get() fails, this hopefully
	 * means that the board generates a suitable master clock some
	 * other way, which is fine by us.
	 *
	 * We need to do this before probing the i2c bus, as the
	 * camera won't ack any messages when it doesn't have a clock.
	 */
	mclk = clk_get(NULL, "mt9v011_mclk");
	if (IS_ERR(mclk)) {
		mclk = NULL;
	} else {
	  // clk_set_rate(mclk, 2000000); /* 2 MHz TODO: might not be correct */

	  // clk_set_rate(mclk, 20000000); /* 20 MHz TODO: might not be correct */
	  
	  clk_set_rate(mclk, 5000000); /* 5 MHz trying lower speed... */
	  clk_enable(mclk);
	}

	return mt9t001_init_cam();

	//	return i2c_add_driver(&mt9t001_driver);
}
module_init(mt9t001_init);

static void __exit mt9t001_exit(void)
{
	if (mclk) {
		clk_disable(mclk);
		clk_put(mclk);
	}
	//	i2c_del_driver(&mt9t001_driver);

	mt9t001_cleanup( is_global );
}
module_exit(mt9t001_exit);

MODULE_AUTHOR("Haavard Skinnemoen <hskinnemoen@atmel.com>, Marius Kotsbak <marius@kotsbak.com>");
MODULE_DESCRIPTION("Micron MT9T001 Image Sensor driver");
MODULE_LICENSE("GPL");
