//
// temp_ctrl.c
//

#include <pthread.h>

#include "adc.h"
#include "gpio.h"

int duty_cycle_on = 0;  // up to regulation_time
#define regulating_time 50 // ms

void *Duty_func(void *threadid)
{
  while(1) {
    if ( duty_cycle_on > 0 ) {
      gpio_set_output( 0 ); // on
      usleep( duty_cycle_on );
    }
    
    if ( duty_cycle_on < regulating_time ) {
      gpio_set_output( 1 ); // off
      usleep( regulating_time - duty_cycle_on );
    }
  }
}


/**
Returns new estimated temperature
**/
float estimate_temp( float outside_temp, float inside_temp, float environment_temp, float secs_elapsed )
{
  const float constant = 0.0239178 / 8; // try and fail...
  const float constant_environment = constant / 3.4; // utifra maaling
  
  float delta_temp_pr_sec = constant * (outside_temp - inside_temp)
    - constant_environment * (inside_temp - environment_temp); // try to compensate for environment temperature conduction

  return inside_temp + delta_temp_pr_sec * secs_elapsed;
}


/** Returns control output **/
float regulate_outer_temp( float heater_temp, float setpoint, float Kp, float Ti ) {
    float paadrag;
    static float lastPaadrag = 0.0;
    
    //float diff = (setpoint - est_temp);
    float diff = (setpoint - heater_temp); // - regulerer etter estimert temp.
    // P-reg
    //    paadrag = Kp * diff; // 0-1.0

    // PI-reg
    paadrag = lastPaadrag + Kp*diff + Ti*diff;
    

    if (paadrag > 1.0) paadrag = 1.0;
    else if (paadrag <= 0) paadrag = 0.0;
    
    int duty_on = regulating_time * paadrag;
    int duty_off = regulating_time - duty_on;

    duty_cycle_on = duty_on; // set global variabel for duty thread


    /*    if ( duty_on > 0.001 ) {
      gpio_set_output( 0 ); // on
      usleep( duty_on );
    }
    
    if ( duty_off > 0.001 ) {
      gpio_set_output( 1 ); // off
      usleep( duty_off );
    }
    */
    return paadrag;
}

int main (int argc, char **argv)
{
  ad7993_init();
  gpio_init();

  printf( "Temperatures: \nReaction chamber, environment, heater\n" );
  
  //#define Kp 0.5

#define Kp 0.1
#define Ti 0.1


#define setpoint 40.0

  pthread_t duty_thread;
  printf("Creating thread\n");
  pthread_create(&duty_thread, NULL, Duty_func, NULL);

  struct timeval tv;
  gettimeofday (&tv, NULL);
  long int startSecs = tv.tv_sec;
  long int startMillis = tv.tv_usec / 1000;

  int i = 0;

  float est_temp = read_temp(1); // Set to real value during testing. Should be set to environment temperature  //read_temp(4);

  while (1 ) {
    float heater_temp = read_temp(4) - 0.5; // calibrating

    float setpoint_outer_temp;
    setpoint_outer_temp = est_temp + ( 3.0 * (setpoint - est_temp) );

    float paadrag = regulate_outer_temp( heater_temp, setpoint_outer_temp, Kp, Ti );

    gettimeofday (&tv, NULL);
    int millis = (tv.tv_sec - startSecs) * 1000 + (tv.tv_usec/1000 - startMillis);

    est_temp = estimate_temp( heater_temp, est_temp, read_temp(3) /*envir.*/, regulating_time / 1000.0 );

    printf ( "%i %f %f %f %f %f\n", millis, read_temp(1), read_temp(3), heater_temp, paadrag, est_temp );

    usleep( 100 );
  }
  
   pthread_exit(NULL);

  return 0;
}
